
/* Chrysalide - Outil d'analyse de fichiers binaires
 * strsym.c - équivalent Python du fichier "format/strsym.h"
 *
 * Copyright (C) 2018-2019 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "strsym.h"


#include <malloc.h>
#include <pygobject.h>


#include <i18n.h>


#include <format/strsym.h>


#include "symbol.h"
#include "../access.h"
#include "../helpers.h"
#include "../arch/feeder.h"



/* Indique si une chaîne de caractères est liée au format. */
static PyObject *py_string_symbol_get_structural(PyObject *, void *);

/* Définit si une chaîne de caractères est liée au format. */
static int py_string_symbol_set_structural(PyObject *, PyObject *, void *);

/* Fournit l'encodage d'une chaîne de caractères. */
static PyObject *py_string_symbol_get_encoding(PyObject *, void *);

/* Fournit la chaîne brute de caractères du symbole. */
static PyObject *py_string_symbol_get_raw(PyObject *, void *);

/* Fournit la chaîne de caractères du symbole. */
static PyObject *py_string_symbol_get_utf8(PyObject *, void *);

/* Définit les constantes pour les chaînes de caractères. */
static bool py_string_symbol_define_constants(PyTypeObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Indique si une chaîne de caractères est liée au format.      *
*                                                                             *
*  Retour      : Indication sur l'emploi de la chaîne.                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_string_symbol_get_structural(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GStrSymbol *symbol;                     /* Elément à consulter         */
    bool status;                            /* Etat à transmettre          */

#define STRING_SYMBOL_STRUCTURAL_ATTRIB PYTHON_GETSET_DEF_FULL                  \
(                                                                               \
    structural, py_string_symbol,                                               \
    "True if the string symbol is linked to the file structure, else False."    \
)

    symbol = G_STR_SYMBOL(pygobject_get(self));

    status = g_string_symbol_is_structural(symbol);

    result = status ? Py_True : Py_False;
    Py_INCREF(result);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                value   = valeur fournie à intégrer ou prendre en compte.    *
*                closure = adresse non utilisée ici.                          *
*                                                                             *
*  Description : Définit si une chaîne de caractères est liée au format.      *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static int py_string_symbol_set_structural(PyObject *self, PyObject *value, void *closure)
{
    int ret;                                /* Bilan d'analyse             */
    GStrSymbol *symbol;                     /* Elément à consulter         */

    ret = PyBool_Check(value);
    if (!ret) return -1;

    symbol = G_STR_SYMBOL(pygobject_get(self));

    g_string_symbol_set_structural(symbol, value == Py_True);

    return 0;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit l'encodage d'une chaîne de caractères.               *
*                                                                             *
*  Retour      : Type d'encodage utilisé.                                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_string_symbol_get_encoding(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GStrSymbol *symbol;                     /* Elément à consulter         */
    StringEncodingType encoding;            /* Encodage associé à la chaîne*/

    symbol = G_STR_SYMBOL(pygobject_get(self));
    encoding = g_string_symbol_get_encoding(symbol);

    result = PyLong_FromLong(encoding);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit la chaîne brute de caractères du symbole.            *
*                                                                             *
*  Retour      : Chaîne de caractères d'origine.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_string_symbol_get_raw(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GStrSymbol *symbol;                     /* Elément à consulter         */
    size_t len;                             /* Taille de la chaîne         */
    const char *data;                       /* Données à manipuler         */

    symbol = G_STR_SYMBOL(pygobject_get(self));
    data = g_string_symbol_get_raw(symbol, &len);

    result = PyBytes_FromStringAndSize(data, len);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit la chaîne de caractères du symbole.                  *
*                                                                             *
*  Retour      : Chaîne de caractères, à priori en UTF-8.                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_string_symbol_get_utf8(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GStrSymbol *symbol;                     /* Elément à consulter         */
    size_t len;                             /* Taille de la chaîne         */
    const char *data;                       /* Données à manipuler         */

    symbol = G_STR_SYMBOL(pygobject_get(self));
    data = g_string_symbol_get_utf8(symbol, &len);

    result = PyUnicode_FromStringAndSize(data, len);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : obj_type = type dont le dictionnaire est à compléter.        *
*                                                                             *
*  Description : Définit les constantes pour les chaînes de caractères.       *
*                                                                             *
*  Retour      : true en cas de succès de l'opération, false sinon.           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool py_string_symbol_define_constants(PyTypeObject *obj_type)
{
    bool result;                            /* Bilan à retourner           */

    result = true;

    result &= PyDict_AddULongMacro(obj_type, SET_NONE);

    result &= PyDict_AddULongMacro(obj_type, SET_ASCII);
    result &= PyDict_AddULongMacro(obj_type, SET_UTF_8);
    result &= PyDict_AddULongMacro(obj_type, SET_MUTF_8);

    result &= PyDict_AddULongMacro(obj_type, SET_GUESS);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_string_symbol_type(void)
{
    static PyMethodDef py_str_symbol_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_str_symbol_getseters[] = {
        STRING_SYMBOL_STRUCTURAL_ATTRIB,
        {
            "encoding", py_string_symbol_get_encoding, NULL,
            "Encoding of the string.", NULL
        },
        {
            "raw", py_string_symbol_get_raw, NULL,
            "String content as raw data.", NULL
        },
        {
            "utf8", py_string_symbol_get_utf8, NULL,
            "String content as UTF-8 data.", NULL
        },
        { NULL }
    };

    static PyTypeObject py_str_symbol_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.format.StrSymbol",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide string symbol",

        .tp_methods     = py_str_symbol_methods,
        .tp_getset      = py_str_symbol_getseters

    };

    return &py_str_symbol_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.format.StrSymbol'.     *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_string_symbol_is_registered(void)
{
    PyTypeObject *type;                     /* Type Python 'StrSymbol'     */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_string_symbol_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.format");

        dict = PyModule_GetDict(module);

        if (!ensure_python_proxy_feeder_is_registered())
            return false;

        if (!register_class_for_pygobject(dict, G_TYPE_STR_SYMBOL, type, get_python_binary_symbol_type()))
            return false;

        if (!py_string_symbol_define_constants(type))
            return false;

    }

    return true;

}
